#include "metric_list.h"
#include "xalloc.h"
#include "license_logging.h"

struct metric_t {
	char *id;
	int value;
	int min;
	int max;
};

struct metric_node_t{
	struct metric_t *metric;
	struct metric_node_t *next;
};

struct metric_list_t {
	struct metric_node_t *head;
};


#define ASSERT_RET(_EXP, _RET) \
	if ((_EXP)) { \
		log_warn("Usage error: invalid parameter ('%s'), returning %s", #_EXP, #_RET); \
		return (_RET); \
	}

struct metric_list_t * metric_list_new(void) {
	struct metric_list_t *list = xcalloc(1, sizeof(struct metric_list_t));
	list->head = NULL;
	return list;
}

static void metric_node_destroy(struct metric_node_t *node) {
	if (node) {
		if(node->metric) {
			xfree(node->metric->id);
			xfree(node->metric);
		}
		xfree(node);
	}
}

void metric_list_destroy(struct metric_list_t *list) {
	if (!list)
		return;

	struct metric_node_t *node = NULL;
	struct metric_node_t *head = list->head;
	if (head != NULL) {
		node = head->next;
		while (node != NULL) {
			struct metric_node_t *next = node->next;
			metric_node_destroy(node);
			node = next;
		}
		metric_node_destroy(head);
	}
	xfree(list);
}

static struct metric_node_t * metric_node_create(const char *id, int value, int min, int max) {
	struct metric_node_t *node = xcalloc(1, sizeof(struct metric_node_t));
	node->metric = xcalloc(1, sizeof(struct metric_t));
	node->metric->id = xstrdup(id);
	node->metric->value = value;
	node->metric->min = min;
	node->metric->max = max;
	node->next = NULL;
	return node;
}

struct metric_node_t * metric_list_add(struct metric_list_t *list, const char *id, int value,
		int min, int max) {
	ASSERT_RET(!list, NULL);

	struct metric_node_t *new_node = NULL;
	new_node = metric_node_create(id, value, min, max);
	if (!list->head)/* uninitialized head node, no need to consider next pointers*/
		list->head = new_node;

	else {
		struct metric_node_t *p = list->head;
		while (p->next != NULL)
			p = p->next;

		p->next = new_node;
	}
	return new_node;
}

struct metric_node_t * metric_list_head(struct metric_list_t *list) {
	ASSERT_RET(!list, NULL);
	ASSERT_RET(!list->head, NULL);
	return list->head;
}

struct metric_node_t * metric_node_next(struct metric_node_t *node) {
	ASSERT_RET(!node, NULL);
	ASSERT_RET(!node->next, NULL);
	return node->next;
}
int metric_node_value(struct metric_node_t *node) {
	ASSERT_RET(!node, 0);
	ASSERT_RET(!node->metric, 0);
	return node->metric->value;
}

int metric_node_min(struct metric_node_t *node) {
	ASSERT_RET(!node, 0);
	ASSERT_RET(!node->metric, 0);
	return node->metric->min;
}

int metric_node_max(struct metric_node_t *node) {
	ASSERT_RET(!node, 0);
	ASSERT_RET(!node->metric, 0);
	return node->metric->max;
}

const char * metric_node_id(struct metric_node_t *node) {
	ASSERT_RET(!node, 0);
	ASSERT_RET(!node->metric, 0);
	return node->metric->id;
}

struct metric_list_t * metric_list_copy(const struct metric_list_t *list) {
	ASSERT_RET(!list, NULL);
	struct metric_list_t *copy = metric_list_new();
	struct metric_node_t *node = NULL;
	node = list->head;
	while (node != NULL) {
		metric_list_add(copy, node->metric->id, node->metric->value,
				node->metric->min, node->metric->max);
		node = node->next;
	}
	return copy;
}
